//
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// N o t e b o o k   H a r d w a r e   C o n t r o l   A C P I   D L L
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// (http://www.pbus-167.com/nhc/nhc.htm)
//
// ACPI file for Asus V6J
// Created on 14.05.2007
// Created by Stingbyte
// File Version: 1.0
//
// Updates: Please visit www.stingbyte.com
//
// v1.0:
// - First public Release
// - Brightness Control working with ability to change brightness
// - Brightness Control set to enabled by default, so that control is working in NHC's Graphic Tab
// - Hardware Control: WLAN-LED ON/OFF, Bluetooth ON/OFF
// - FAN: manual FAN ON/OFF with Speed Setting
// - Automatic FAN Control: FAN ON/OFF with Temperature Settings
// - QFAN: automatic fan speed setting according to desired CPU temperature
// - SFAN: intelligent FAN Speed Control with automatic fan speed according to CPU temperature
// - TFAN: intelligent FAN Speed Control with automatic fan speed accordding to Thermal Zones (enabled by default)
//
// Known Bugs/Limitations:
// - Be careful: It is possible to use more than one FAN Control simultaneously by setting them
//   to enabled.
// - Hardware Control: WLAN only switches the LED ON/OFF, not the hardware itself.
//   Bluetooth ON/OFF switches LED & hardware.
// - No sound control yet: Could not find the corresponding methods.
// 
// The CPU temperature is directly read from _TZ.GTMP in C value.
// The FAN Speed is read from _SB.PCI0.SBRG.EC0.CDT1 range 80-255, 0-80=off
//
// CAUTION: Disabling ACPI control at a moment where the FAN is off can
// lead to the situation that the FAN does not turn on at all anymore.
//
// ****!!NOTE:IF YOU TURN OFF YOUR FAN MANUALLY, OVERHEATING MAY OCCUR!!****
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
// Feel free to modify as you wish, but give credit, where credit is due.
//
//  2007, released by stingbyte.com under the terms
// and conditions of the CREATIVE COMMONS LICENSE
// http://creativecommons.org/licenses/by-nc-sa/3.0/
//
// Credits to: Manfred Jaider, cogitandumest, Piotr Kocybik, Eelco Mossel, Pauli Lehto, volker & others forgotten to name here

public static class ASUSTeK 
{

	#region Notebook ASUS V6J : class V6J
    //
    // Class Author: Stingbyte (www.stingbyte.com)
    //
    public static class V6J
    {
        #region Class for ASUS V6J LCD Brightness Control : class BRIGHTNESS
        //
        // The class must have the name BRIGHTNESS
        // original taken from A8Fm, no changes
        public static class BRIGHTNESS
        {
            // Default constants for the brightness class
            public const string description = "LCD Brightness Control";             // Description of current class
            public const bool default_enable = true;                                // true = By default the current class is enabled in NHC
            public const int icon = 1;                                              // 0 = Hardware icon, 1 = Brightness icon, 2 = Fan icon, 3 = Battery icon, 4 = Audio icon

            // Constants for the brightness control (NHC will need this constants)
            public const int minimum_brightness_level = 0;
            public const int maximum_brightness_level = 15;
            public const int brightness_level_step = 1;

            #region Value class for brightness_level : class BRIGHTNESS_LEVEL
            //
            public static class BRIGHTNESS_LEVEL
            {
                // Properties of the value class
                public const string description = "Current LCD Brightness Level";   // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int brightness_level = 15; 							// It must have the same name as the class (case are ignored)
                
                // With this class it is possible to change the value in NHC on runtime
				public static class NHC_WRITE
				{
					public const bool nhc_write = true;				// true -> it is possible to change the value in NHC
					public const int nhc_minimum_value = 0;				// the minimum value of wlan_status
					public const int nhc_maximum_value = 15;				// the maximum value of wlan_status
					public const int nhc_default_value = 15;				// the default value of wlan_status
					public static bool nhc_on_write()	
					{
						return set_brightness(brightness_level);
					}
				}
            }
            //
            #endregion

            // Function to set the brightness (it must have the name set_brightness)
            public static bool set_brightness(int new_level)
            {
                return ACPI.METHOD.SetValue("_SB.ATKD.SPLV", new_level);
            }

            // Function to get and refresh the brightness level (it must have the name get_brightness)
            public static bool get_brightness(ref int level)
            {
                bool result = ACPI.METHOD.GetValue("_SB.ATKD.GPLV", ref level);     // get the brightness level
                BRIGHTNESS_LEVEL.brightness_level = level;
                return result;
            }
        }
        //
        #endregion

        #region Class for ASUS V6J Hardware Control : class HARDWARE
        //original from A8Fm
        public static class HARDWARE
        {
            // Info
            public const string description = "Hardware Control";                   // Description of current class
            public const bool default_enable = false;                               // false = By default the current class is disabled in NHC
            public const int icon = 0;                                              // 0 = Hardware icon, 1 = Brightness icon, 2 = Fan icon, 3 = Battery icon, 4 = Audio icon
            public static int refreshdivider = 1;

            #region Value class for WLAN_CONTROL : class WLAN_CONTROL
            //Currently only WLAN LED under Control - not the card it self
            public static class WLAN_CONTROL
            {
                // Properties of the value class
                public const string description = "Wireless LAN LED";                   // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int wlan_control = 1;                                 // It must have the same name as the class (case are ignored)
                public const string unit = "OFF|ON";                                // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 0;                         // the minimum value of wlan_status
                    public const int nhc_maximum_value = 1;                         // the maximum value of wlan_status
                    public const int nhc_default_value = 0;                         // the default value of wlan_status
                    public static bool initialisation = false;
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
                    	if (!initialisation)
                        {
 							int check = 0;
                    		ACPI.METHOD.GetValue("_SB.ATKD.RSTS", ref check);
                            initialisation = true;
                            if (check == 0 || check == 2)
                            	wlan_control = 0;
                            else
                            	wlan_control = 1;
                            return true;
                        }
                    	if(wlan_control == 0)
                        {
                            return ACPI.METHOD.SetValue("_SB.ATKD.WLED", 0);        // turn off the WLAN
                        }
                        else
                        {
                            return ACPI.METHOD.SetValue("_SB.ATKD.WLED", 1);        // turn on the WLAN
                        }
                    }
                }
            }
            //
            #endregion
			
            #region Value class for BLUETOOTH_CONTROL : class BLUETOOTH_CONTROL
            //
            public static class BLUETOOTH_CONTROL
            {
                // Properties of the value class
                public const string description = "Bluetooth";                      // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int bluetooth_control = 1;                            // It must have the same name as the class (case are ignored)
                public const string unit = "OFF|ON";                                // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 0;                         // the minimum value of wlan_status
                    public const int nhc_maximum_value = 1;                         // the maximum value of wlan_status
                    public const int nhc_default_value = 0;                         // the default value of wlan_status
                    public static bool initialisation = false;
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {	
                    	if (!initialisation)
                        {
 							int check = 0;
                    		ACPI.METHOD.GetValue("_SB.ATKD.RSTS", ref check);
                            initialisation = true;
                            if (check == 0 || check == 1)
                            	bluetooth_control = 0;
                            else
                            	bluetooth_control = 1;
                            return true;
                        }
                    	if(bluetooth_control == 0)
                        {
                            return ACPI.METHOD.SetValue("_SB.ATKD.BLED", 0);        // turn off the bluetooth
                        }
                        else
                        {   
                            return ACPI.METHOD.SetValue("_SB.ATKD.BLED", 1);        // turn on the bluetooth
                        }
                    }
                }
                
            }
            //
            #endregion
        	
        	#region Class for Bluetooth & WLAN Status Refresh : class HARDWARE
            public static bool refresh()
            	{
                	if (HARDWARE.refreshdivider < UpdateSpeed.updatespeed)
                	{
                    	HARDWARE.refreshdivider++;
                    	return true;
                	}
                	int check = 0;
                    ACPI.METHOD.GetValue("_SB.ATKD.RSTS", ref check);
                    if (check == 0 || check == 1)
                    	BLUETOOTH_CONTROL.bluetooth_control = 0;
                    else
                        BLUETOOTH_CONTROL.bluetooth_control = 1;
                    if (check == 0 || check == 2)
                        WLAN_CONTROL.wlan_control = 0;
                    else
                        WLAN_CONTROL.wlan_control = 1;
    	           	HARDWARE.refreshdivider = 1;
                	return true;
            	}
            #endregion
            
            #region Value class for UpdateSpeed : class UpdateSpeed
            public static class UpdateSpeed
            {
                public const string description = "Update WLAN && Bluetooth - Status every";
                public const bool nhc_visible = true;
                public static int updatespeed = 0;
                public const string unit = " second(s)";

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 1;
                    public const int nhc_maximum_value = 20;
                    public const int nhc_default_value = 3;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion
        }
        #endregion
        
        #region Class for Asus V6J FAN Control : class FAN
        //original taken from old A6VA ACPI file, , modified for new V6J ACPI file
        public static class FAN
        {
            public const string description = "Manual FAN Control";
            public const bool default_enable = false;
            public const int icon = 2;

            public static class FAN_ON_OFF
            {
                public const string description = "Current FAN State (on/off)";
                public const bool nhc_visible = true;
                public static int fan_on_off = 0;
                public const string unit = "OFF|ON";

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 1;
                    public const int nhc_default_value = 1;
                    public static bool initialisation = false;

                    public static bool nhc_on_write()
                    {
                        if (!initialisation)
                        {
                            int a = 0;
                            ACPI.NAME.Read("_TZ.PLCY", ref a);
                            FAN_ON_OFF.fan_on_off = ((a == 0) ? 1 : 0);
                            initialisation = true;
                            return true;
                        }
                        if (fan_on_off != 0)
                        {
                            ACPI.NAME.Write("_TZ.PLCY", 0);
                            int a, b, c, res;
                            a = 0x84;
                            b = FAN_SPEED.fan_speed;
                            c = 0xC4;
                            res = ((a << 16) + (b << 8) + (c));
                            ACPI.METHOD.SetValue("_TZ.TMSS", 0);
                            ACPI.METHOD.SetValue("_SB.ATKD.ECRW", res);
                            ACPI.METHOD.Call("_TZ.UMIT");
                        }
                        else
                        {
                            ACPI.NAME.Write("_TZ.PLCY", 1);
                            ACPI.METHOD.Call("_TZ.TCHG");
                        }
                        return true;
                    }
                }
            }
            public static class FAN_SPEED
            {
                public const string description = "Current FAN ON Speed (80-255)";
                public const bool nhc_visible = true;
                public static int fan_speed = 0;
                public const string unit = "";

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 80;
                    public const int nhc_maximum_value = 255;
                    public const int nhc_default_value = 160;
                    public static bool initialisation = false;

                    public static bool nhc_on_write()
                    {
                        if (!initialisation)
                        {
                            ACPI.FIELD.Read("_SB.PCI0.SBRG.EC0.CDT1", ref FAN_SPEED.fan_speed);
                            initialisation = true;
                            return true;
                        }
                        int is_fan_off = 0;
                        ACPI.NAME.Read("_TZ.PLCY", ref is_fan_off);
                        if (is_fan_off == 0)
                        {
                            int a, b, c, res;
                            a = 0x84;
                            b = fan_speed;
                            c = 0xC4;
                            res = ((a << 16) + (b << 8) + (c));
                            ACPI.METHOD.SetValue("_TZ.TMSS", 0);
                            ACPI.METHOD.SetValue("_SB.ATKD.ECRW", res);
                            ACPI.METHOD.Call("_TZ.UMIT");
                        }
                        return true;
                    }
                }
            }
        }
        #endregion
        
        #region Class for Asus V6J Automatic FAN Control : class AUTOMATIC_FAN
        //original taken from A8J, modified for V6J
        public static class AUTOMATIC_FAN
        {
            // true = By default the current class is enabled in NHC
            public const bool default_enable = false;
            // Description of current class
            public const string description = "ACPI Automatic FAN Control";
            // 0 = Hardware icon, 1 = Brightness icon, 2 = Fan icon, 3 = Battery icon, 4 = Audio icon
            public const int icon = 2;

            public static int FAN_ON_TEMP = 55;
            public static int FAN_OFF_TEMP = 38;
            public static int refreshdivider = 1;
            const int BUILT_IN_FAN_OFF_TEMP = 39;

            #region Value class for CPUTEMP_VALUE : class CPUTEMP_VALUE
            //
            public static class CPUTEMP_VALUE
            {
                // Properties of the value class
                public const string description = "Current CPU Temperature";
                public const bool nhc_visible = true;
                // Value
                public static int CPUTEMP_value = 0;                                // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // Unit of value
            }
            //
            #endregion
            
            #region Value class for UpdateSpeed : class UpdateSpeed
            public static class UpdateSpeed
            {
                public const string description = "Update once in";
                public const bool nhc_visible = true;
                public static int updatespeed = 0;
                public const string unit = " second(s)";

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 1;
                    public const int nhc_maximum_value = 20;
                    public const int nhc_default_value = 3;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for FANSTATE : class FANSTATE
            //
            public static class FANSTATE
            {
                // Properties of the value class
                public const string description = "Current FAN State (on/off)";
                public const bool nhc_visible = true;
                // Value
                public static int fanstate = 0;                                // It must have the same name as the class (case are ignored)
                public const string unit = "OFF|ON";                		   // Unit of value
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 1;
                    public const int nhc_default_value = 1;
                    public static bool initialisation = false;

                    public static bool nhc_on_write()
                    {
                        if (!initialisation)
                        {
                            int a = 0;
                        	ACPI.NAME.Read("_TZ.PLCY", ref a);
                            FANSTATE.fanstate = ((a == 0) ? 1 : 0);
                            initialisation = true;
                            return true;
                        }
                        if (FANSTATE.fanstate != 0)
                        {
                            ACPI.NAME.Write("_TZ.PLCY", 0);
                            int a, b, c, res;
                            a = 0x84;
                            b = FAN_SPEED.fan_speed;
                            c = 0xC4;
                            res = ((a << 16) + (b << 8) + (c));
                            ACPI.METHOD.SetValue("_TZ.TMSS", 0);
                            ACPI.METHOD.SetValue("_SB.ATKD.ECRW", res);
                            ACPI.METHOD.Call("_TZ.UMIT");
                        }
                        else
                        {
                            ACPI.NAME.Write("_TZ.PLCY", 1);
                            ACPI.METHOD.Call("_TZ.TCHG");
                        }
                        return true;
                    }
                }
            }
            
            //
            #endregion
            
            #region Value class for FAN_SPEED : class FAN_SPEED
            //
            public static class FAN_SPEED
            {
                public const string description = "Current FAN ON Speed (80-255)";
                public const bool nhc_visible = true;
                public static int fan_speed = 0;
                public const string unit = "";

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 80;
                    public const int nhc_maximum_value = 255;
                    public const int nhc_default_value = 160;
                    public static bool initialisation = false;

                    public static bool nhc_on_write()
                    {
                        if (!initialisation)
                        {
                            int a = 0;
                            ACPI.FIELD.Read("_SB.PCI0.SBRG.EC0.CDT1", ref a);
                            FAN_SPEED.fan_speed = a;
                            initialisation = true;
                            return true;
                        }
                        int is_fan_off = 0;
                        ACPI.NAME.Read("_TZ.PLCY", ref is_fan_off);
                        if (is_fan_off == 0)
                        {
                            int a, b, c, res;
                            a = 0x84;
                            b = fan_speed;
                            c = 0xC4;
                            res = ((a << 16) + (b << 8) + (c));
                            ACPI.METHOD.SetValue("_TZ.TMSS", 0);
                            ACPI.METHOD.SetValue("_SB.ATKD.ECRW", res);
                            ACPI.METHOD.Call("_TZ.UMIT");
                        }
                        return true;
                    }
                }
            }
			//
            #endregion
            
            #region Value class for FANON_VALUE : class FANON_VALUE
            //
            public static class FANON_VALUE
            {
                // Properties of the value class
                public const string description = "FAN ON CPU Temperature";
                public const bool nhc_visible = true;

                // Value
                public static int FANON_value = FAN_ON_TEMP;                        // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // Unit of value

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 35;                        // minimum input value
                    public const int nhc_maximum_value = 75;                        // maximum input value
                    public const int nhc_default_value = 55;                        // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
                        if (FANON_value <= FAN_OFF_TEMP)
                        {
                            FANON_value = FAN_OFF_TEMP + 1;
                        }
                        FAN_ON_TEMP = FANON_value;
                        return true;
                    }
                }
            }
            //
            #endregion

            #region Value class for FANOFF_VALUE : class FANOFF_VALUE
            //
            public static class FANOFF_VALUE
            {
                // Properties of the value class
                public const string description = "FAN OFF CPU Temperature";
                public const bool nhc_visible = true;

                // Value
                public static int FANOFF_value = FAN_OFF_TEMP;                                // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // Unit of value

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 30;                        // minimum input value
                    public static int nhc_maximum_value = 60;                        // maximum input value
                    public const int nhc_default_value = 45;                        // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
                        if (FANOFF_value >= FAN_ON_TEMP)
                        {
                            FANOFF_value = FAN_ON_TEMP - 1;
                        }
                        FAN_OFF_TEMP = FANOFF_value;
                        return true;
                    }
                }
            }
            //
            #endregion
			public const double refresh_interval = 1;
            public static bool refresh()
            {
                if (AUTOMATIC_FAN.refreshdivider < UpdateSpeed.updatespeed)
                {
                    AUTOMATIC_FAN.refreshdivider++;
                    return true;
                }
                AUTOMATIC_FAN.refreshdivider = 1;
                ACPI.FIELD.Read("_TZ.GTMP", ref CPUTEMP_VALUE.CPUTEMP_value);
                if (FANSTATE.fanstate == 1)
                {
                    if (CPUTEMP_VALUE.CPUTEMP_value < FAN_OFF_TEMP)
                    {
                        ACPI.NAME.Write("_TZ.PLCY", (byte)1);
                        ACPI.METHOD.Call("_TZ.TCHG");
                        FANSTATE.fanstate = 0;
                    }
                }
                //FAN state OFF
                else if (FANSTATE.fanstate == 0)
                {
                    if (CPUTEMP_VALUE.CPUTEMP_value > FAN_ON_TEMP)
                    {
                        int a, b, c, res;
                        a = 0x84;
                        b = FAN_SPEED.fan_speed;
                        c = 0xC4;
                        res = ((a << 16) + (b << 8) + (c));
                        ACPI.METHOD.SetValue("_TZ.TMSS", 0);
                        ACPI.METHOD.SetValue("_SB.ATKD.ECRW", res);
                        ACPI.METHOD.Call("_TZ.UMIT");
                        FANSTATE.fanstate = 1;
                    }
                }
                return true;
            }
        }
        //
        #endregion
        
        #region Class for Asus V6J QFAN Control : class QFAN
        //original taken from A8Fm, , modified for V6J
        public static class QFAN
        {
            public const string description = "ACPI Automatic QuietFAN Temperature Control";
            public const bool default_enable = false;
            public const int icon = 2;

            #region Value class for QFAN_CPUTemperature : class QFAN_CPUTemperature
            public static class QFAN_CPUTemperature
            {
                public const string description = "Current CPU Temperature";
                public const bool nhc_visible = true;
                public static int qfan_cputemperature = 0;
                public const string unit = "C";
            }
            #endregion

            #region Value class for QFAN_FanSpeed : class QFAN_FanSpeed
            public static class QFAN_FanSpeed
            {
                public const string description = "Current FAN Speed (80-255, 0=off)";
                public const bool nhc_visible = true;
                public static int qfan_fanspeed = 0;
                public const string unit = "";
            }
            #endregion
            
            #region Value class for QFAN_UpdateSpeed : class QFAN_UpdateSpeed
            public static class QFAN_UpdateSpeed
            {
                public const string description = "Update once in";
                public const bool nhc_visible = true;
                public static int QFAN_updatespeed = 0;
                public const string unit = " second(s)";

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 1;
                    public const int nhc_maximum_value = 20;
                    public const int nhc_default_value = 3;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for QFAN_CPUTemperaturePrevious : class QFAN_CPUTemperaturePrevious
            public static class QFAN_CPUTemperaturePrevious
            {
                public const string description = "";
                public const bool nhc_visible = false;
                public static int qfan_cputemperatureprevious = -1;
                public const string unit = "";
                public static int qfan_refreshdivider = 1;
            }
            #endregion

            #region Value class for QFAN_CPUTemperatureDesired : class QFAN_CPUTemperatureDesired
            public static class QFAN_CPUTemperatureDesired
            {
                public const string description = "Desired CPU Temperature";
                public const bool nhc_visible = true;
                public static int qfan_cputemperaturedesired = 0;
                public const string unit = "C";

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 80;
                    public const int nhc_default_value = 45;
                    public static bool initialisation = false;

                    public static bool nhc_on_write()
                    {
                        if (!initialisation)
                        {
                            ACPI.FIELD.Read("_TZ.GTMP", ref QFAN_CPUTemperature.qfan_cputemperature);
                            QFAN_CPUTemperaturePrevious.qfan_cputemperatureprevious = QFAN_CPUTemperature.qfan_cputemperature;
                            ACPI.FIELD.Read("_SB.PCI0.SBRG.EC0.CDT1", ref QFAN_FanSpeed.qfan_fanspeed);
                            initialisation = true;
                            return true;
                        }
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for QFAN_CPUFanSpeedDesired : class QFAN_CPUFanSpeedDesired
            public static class QFAN_CPUFanSpeedDesired
            {
                public const string description = "Desired FAN Speed";
                public const bool nhc_visible = true;
                public static int qfan_cpufanspeeddesired = 0;
                public const string unit = "";

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 255;
                    public const int nhc_default_value = 144;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for QFAN_CPUFanSpeedStep : class QFAN_CPUFanSpeedStep
            public static class QFAN_CPUFanSpeedStep
            {
                public const string description = "FAN Speed Step";
                public const bool nhc_visible = true;
                public static int qfan_cpufanspeedstep = 1;
                public const string unit = "";

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 1;
                    public const int nhc_maximum_value = 30;
                    public const int nhc_default_value = 1;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            public const double refresh_interval = 1;
            public static bool refresh()
            {
                if (QFAN_CPUTemperaturePrevious.qfan_refreshdivider < 3)
                {
                    QFAN_CPUTemperaturePrevious.qfan_refreshdivider++;
                    return true;
                }
                QFAN_CPUTemperaturePrevious.qfan_refreshdivider = 1;
                int temperature_current = 0;
                int temperature_desired = 0;
                int speed_current = 0;
                int speed_previous = 0;
                int speed_desired = 0;
                int speed_step = 0;

                temperature_desired = QFAN_CPUTemperatureDesired.qfan_cputemperaturedesired;
                speed_desired = QFAN_CPUFanSpeedDesired.qfan_cpufanspeeddesired;
                speed_step = QFAN_CPUFanSpeedStep.qfan_cpufanspeedstep;
                ACPI.FIELD.Read("_TZ.GTMP", ref temperature_current);
                ACPI.FIELD.Read("_SB.PCI0.SBRG.EC0.CDT1", ref speed_current);

                if (QFAN_CPUTemperaturePrevious.qfan_cputemperatureprevious != -1)
                {
                    if (temperature_current != temperature_desired)
                    {
                        if (temperature_current > temperature_desired)
                        {
                            if (temperature_current == QFAN_CPUTemperaturePrevious.qfan_cputemperatureprevious)
                            {
                                speed_previous = speed_current;
                                speed_current += speed_step;
                                if (speed_current > 255) speed_current = 255;
                                if (speed_current != speed_previous) QFAN_SetCPUFanSpeed(speed_current);
                            }
                        }
                        else
                        {
                            if (QFAN_CPUTemperaturePrevious.qfan_cputemperatureprevious == temperature_current)
                            {
                                if (speed_current > speed_desired)
                                {
                                    speed_previous = speed_current;
                                    speed_current -= speed_step;
                                    if (speed_current < 0) speed_current = 0;
                                    if (speed_current != speed_previous) QFAN_SetCPUFanSpeed(speed_current);
                                }
                            }
                        }
                    }
                }

                QFAN_CPUTemperature.qfan_cputemperature = temperature_current;
                QFAN_FanSpeed.qfan_fanspeed = speed_current;
                QFAN_CPUTemperaturePrevious.qfan_cputemperatureprevious = temperature_current;

                return true;
            }

            private static bool QFAN_SetCPUFanSpeed(int speed)
            {
                ACPI.NAME.Write("_TZ.PLCY", 0);
                int a, b, c, res;
                a = 0x84;
                b = speed;
                c = 0xC4;
                res = ((a << 16) + (b << 8) + (c));
                ACPI.METHOD.SetValue("_TZ.TMSS", 0);
                ACPI.METHOD.SetValue("_SB.ATKD.ECRW", res);
                ACPI.METHOD.Call("_TZ.UMIT");
                return true;
            }
        }
        #endregion
        
        #region Class for Asus V6J SFAN Control : class SFAN
        //original taken from A8JP, modified for V6J
        public static class SFAN
        {
            public const string description = "ACPI Intelligent FAN Speed Control";
            public const bool default_enable = false;
            public const int icon = 2;

            #region Value class for SFAN_CPUTemperature : class SFAN_CPUTemperature
            public static class SFAN_CPUTemperature
            {
                public const string description = "Current CPU Temperature";
                public const bool nhc_visible = true;
                public static int sfan_cputemperature = 0;
                public const string unit = "C";
            }
            #endregion

            #region Value class for SFAN_FanSpeed : class SFAN_FanSpeed
            public static class SFAN_FanSpeed
            {
                public const string description = "Current FAN Speed (80-255, 0=off)";
                public const bool nhc_visible = true;
                public static int sfan_fanspeed = 0;
                public const string unit = "";
                public static int sfan_refreshdivider = 1;
            }
            #endregion

            #region Value class for SFAN_UpdateSpeed : class SFAN_UpdateSpeed
            public static class SFAN_UpdateSpeed
            {
                public const string description = "Update once in";
                public const bool nhc_visible = true;
                public static int sfan_updatespeed = 0;
                public const string unit = " second(s)";
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 1;
                    public const int nhc_maximum_value = 20;
                    public const int nhc_default_value = 3;
                    public static bool nhc_on_write()
                    {
                        return true;		// nothing to do; value used only by timer
                    }
                }
            }
            #endregion

            #region Value class for SFAN_MaxTemperature : class SFAN_MaxTemperature
            public static class SFAN_MaxTemperature
            {
                public const string description = "Maximum FAN Speed CPU Temperature";
                public const bool nhc_visible = true;
                public static int sfan_maxtemperature = 0;
                public const string unit = "C";
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 65;
                    public const int nhc_maximum_value = 85;
                    public const int nhc_default_value = 75;
                    public static bool nhc_on_write()
                    {
                        return true;		// nothing to do; value used only by timer
                    }
                }
            }
            #endregion

            #region Value class for SFAN_MinTemperature : class SFAN_MinTemperature
            public static class SFAN_MinTemperature
            {
                public const string description = "FAN ON CPU Temperature";
                public const bool nhc_visible = true;
                public static int sfan_mintemperature = 0;
                public const string unit = "C";
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 35;
                    public const int nhc_maximum_value = 55;
                    public const int nhc_default_value = 45;
                    public static bool nhc_on_write()
                    {
                        return true;		// nothing to do; value used only by timer
                    }
                }
            }
            #endregion

            public static bool refresh()
            {
                if (SFAN_FanSpeed.sfan_refreshdivider < SFAN_UpdateSpeed.sfan_updatespeed)		// delay-loop
                {
                    SFAN_FanSpeed.sfan_refreshdivider++;
                    return true;
                }
                SFAN_FanSpeed.sfan_refreshdivider = 1;
                int temperature_current = 0;
                int speed_current = 0;
                int temperature_min = SFAN_MinTemperature.sfan_mintemperature;
                int temperature_max = SFAN_MaxTemperature.sfan_maxtemperature;
                ACPI.FIELD.Read("_TZ.GTMP", ref temperature_current); 
                ACPI.FIELD.Read("_SB.PCI0.SBRG.EC0.CDT1", ref speed_current);
                int fan_speed = (temperature_current - temperature_min) * 175 / (temperature_max - temperature_min) + 80;		// calculating fan_speed
                if (fan_speed > 79)		// fan starts spinning only > 75
                {
                    if (fan_speed != speed_current) SFAN_SetCPUFanSpeed(fan_speed);		// to avoid useless updating
                }
                else
                {
                    if (speed_current != 0)		// to avoid useless updating
                    {
                        SFAN_SetCPUFanSpeed(0);
                        ACPI.NAME.Write("_TZ.PLCY", 1);
                    }
                }
                SFAN_CPUTemperature.sfan_cputemperature = temperature_current;
                SFAN_FanSpeed.sfan_fanspeed = speed_current;
                return true;
            }

            private static bool SFAN_SetCPUFanSpeed(int speed)		// the speed-setting-thing
            {
                int a, b, c, res;
                a = 0x84;
                b = speed;
                c = 0xC4;
                res = ((a << 16) + (b << 8) + (c));
                ACPI.METHOD.SetValue("_TZ.TMSS", 0);
                ACPI.METHOD.SetValue("_SB.ATKD.ECRW", res);
                ACPI.METHOD.Call("_TZ.UMIT");
                return true;
            }
        }
        #endregion

        #region Class for Asus V6J TFAN Control : class TFAN
		//original taken from A8Fm (SFAN), modified for V6J
        public static class TFAN
        {
            public const string description = "ACPI Intelligent FAN Speed Control (Thermal Zones)";
            public const bool default_enable = true;
            public const int icon = 2;

            #region Value class for TFAN_CPUTemperature : class TFAN_CPUTemperature
            public static class TFAN_CPUTemperature
            {
                public const string description = "Current CPU Temperature";
                public const bool nhc_visible = true;
                public static int TFAN_cputemperature = 0;
                public const string unit = "C";
            }
            #endregion

            #region Value class for TFAN_FanSpeed : class TFAN_FanSpeed
            public static class TFAN_FanSpeed
            {
                public const string description = "Current FAN Speed (80-255, 0=off)";
                public const bool nhc_visible = true;
                public static int TFAN_fanspeed = 0;
                public const string unit = "";
                public static int TFAN_refreshdivider = 0;
            }
            #endregion

            #region Value class for TFAN_UpdateSpeed : class TFAN_UpdateSpeed
            public static class TFAN_UpdateSpeed
            {
                public const string description = "Update once in";
                public const bool nhc_visible = true;
                public static int TFAN_updatespeed = 0;
                public const string unit = " second(s)";

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 1;
                    public const int nhc_maximum_value = 20;
                    public const int nhc_default_value = 3;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfile1 : class TFAN_SpeedProfile1
            public static class TFAN_SpeedProfile1
            {
                public const string description = "Profile 1, the highest CPU Temperature allowed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofile1 = 0;
                public const string unit = "C";
                public static bool initialisation = false;

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 100;
                    public const int nhc_default_value = 45;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfile1_S : class TFAN_SpeedProfile1_S
            public static class TFAN_SpeedProfile1_S
            {
                public const string description = "Profile 1, FAN Speed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofile1_s = 0;
                public const string unit = "";
                public static bool initialisation = false;

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 255;
                    public const int nhc_default_value = 0;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfile2 : class TFAN_SpeedProfile2
            public static class TFAN_SpeedProfile2
            {
                public const string description = "Profile 2, the highest CPU Temperature allowed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofile2 = 0;
                public const string unit = "C";
                public static bool initialisation = false;

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 100;
                    public const int nhc_default_value = 55;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfile2_S : class TFAN_SpeedProfile2_S
            public static class TFAN_SpeedProfile2_S
            {
                public const string description = "Profile 2, FAN Speed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofile2_s = 0;
                public const string unit = "";
                public static bool initialisation = false;

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 255;
                    public const int nhc_default_value = 80;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfile3 : class TFAN_SpeedProfile3
            public static class TFAN_SpeedProfile3
            {
                public const string description = "Profile 3, the highest CPU Temperature allowed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofile3 = 0;
                public const string unit = "C";
                public static bool initialisation = false;

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 100;
                    public const int nhc_default_value = 65;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfile3_S : class TFAN_SpeedProfile3_S
            public static class TFAN_SpeedProfile3_S
            {
                public const string description = "Profile 3, FAN Speed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofile3_s = 0;
                public const string unit = "";
                public static bool initialisation = false;

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 255;
                    public const int nhc_default_value = 100;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfile4 : class TFAN_SpeedProfile4
            public static class TFAN_SpeedProfile4
            {
                public const string description = "Profile 4, the highest CPU Temperature allowed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofile4 = 0;
                public const string unit = "C";
                public static bool initialisation = false;

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 100;
                    public const int nhc_default_value = 75;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfile4_S : class TFAN_SpeedProfile4_S
            public static class TFAN_SpeedProfile4_S
            {
                public const string description = "Profile 4, FAN Speed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofile4_s = 0;
                public const string unit = "";
                public static bool initialisation = false;

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 255;
                    public const int nhc_default_value = 140;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfile5 : class TFAN_SpeedProfile5
            public static class TFAN_SpeedProfile5
            {
                public const string description = "Profile 5, the highest CPU Temperature allowed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofile5 = 0;
                public const string unit = "C";
                public static bool initialisation = false;

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 100;
                    public const int nhc_default_value = 85;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfile5_S : class TFAN_SpeedProfile5_S
            public static class TFAN_SpeedProfile5_S
            {
                public const string description = "Profile 5, FAN Speed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofile5_s = 0;
                public const string unit = "";
                public static bool initialisation = false;

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 255;
                    public const int nhc_default_value = 180;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfile6 : class TFAN_SpeedProfile6
            public static class TFAN_SpeedProfile6
            {
                public const string description = "Profile 6, the highest CPU Temperature allowed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofile6 = 0;
                public const string unit = "C";
                public static bool initialisation = false;

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 100;
                    public const int nhc_default_value = 100;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfile6_S : class TFAN_SpeedProfile6_S
            public static class TFAN_SpeedProfile6_S
            {
                public const string description = "Profile 6, FAN Speed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofile6_s = 0;
                public const string unit = "";
                public static bool initialisation = false;

                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;
                    public const int nhc_minimum_value = 0;
                    public const int nhc_maximum_value = 255;
                    public const int nhc_default_value = 255;

                    public static bool nhc_on_write()
                    {
                        // nothing to do; value used only by timer
                        return true;
                    }
                }
            }
            #endregion

            #region Value class for TFAN_SpeedProfileDefault : class TFAN_SpeedProfileDefault
            public static class TFAN_SpeedProfileDefault
            {
                public const string description = "Default Profile, FAN Speed";
                public const bool nhc_visible = true;
                public static int TFAN_speedprofiledefault = 255;
                public const string unit = "";
                public static bool initialisation = false;
            }
            #endregion

            public const double refresh_interval = 1;
            public static bool refresh()
            {
                if (TFAN_FanSpeed.TFAN_refreshdivider < TFAN_UpdateSpeed.TFAN_updatespeed)
                {
                    TFAN_FanSpeed.TFAN_refreshdivider++;
                    return true;
                }
                TFAN_FanSpeed.TFAN_refreshdivider = 1;

                int temperature_current = 0;
                int speed_current = 0;

                ACPI.FIELD.Read("_TZ.GTMP", ref temperature_current);
                ACPI.FIELD.Read("_SB.PCI0.SBRG.EC0.CDT1", ref speed_current);

                int[] t_ = new int[6];
                t_[0] = TFAN_SpeedProfile1.TFAN_speedprofile1;
                t_[1] = TFAN_SpeedProfile2.TFAN_speedprofile2;
                t_[2] = TFAN_SpeedProfile3.TFAN_speedprofile3;
                t_[3] = TFAN_SpeedProfile4.TFAN_speedprofile4;
                t_[4] = TFAN_SpeedProfile5.TFAN_speedprofile5;
                t_[5] = TFAN_SpeedProfile6.TFAN_speedprofile6;

                int[] s_ = new int[6];
                s_[0] = TFAN_SpeedProfile1_S.TFAN_speedprofile1_s;
                s_[1] = TFAN_SpeedProfile2_S.TFAN_speedprofile2_s;
                s_[2] = TFAN_SpeedProfile3_S.TFAN_speedprofile3_s;
                s_[3] = TFAN_SpeedProfile4_S.TFAN_speedprofile4_s;
                s_[4] = TFAN_SpeedProfile5_S.TFAN_speedprofile5_s;
                s_[5] = TFAN_SpeedProfile6_S.TFAN_speedprofile6_s;

                int tmp;
                for (int i = 1; i < 6; i++)
                    for (int j = 5; j > 0; j--)
                        if (t_[j - 1] > t_[j])
                        {
                            tmp = t_[j];
                            t_[j] = t_[j - 1];
                            t_[j - 1] = tmp;
                            tmp = s_[j];
                            s_[j] = s_[j - 1];
                            s_[j - 1] = tmp;
                        }

                bool ok = false;
                for (int i = 0; i < 6; i++)
                    if (temperature_current <= t_[i])
                    {
                        if (s_[i] != speed_current) TFAN_SetCPUFanSpeed(s_[i]);
                        ok = true;
                        break;
                    }

                if (!ok) TFAN_SetCPUFanSpeed(255);

                TFAN_CPUTemperature.TFAN_cputemperature = temperature_current;
                TFAN_FanSpeed.TFAN_fanspeed = speed_current;

                return true;
            }

            private static bool TFAN_SetCPUFanSpeed(int speed)
            {
                ACPI.NAME.Write("_TZ.PLCY", 0);
                int a, b, c, res;
                a = 0x84;
                b = speed;
                c = 0xC4;
                res = ((a << 16) + (b << 8) + (c));
                ACPI.METHOD.SetValue("_TZ.TMSS", 0);
                ACPI.METHOD.SetValue("_SB.ATKD.ECRW", res);
                ACPI.METHOD.Call("_TZ.UMIT");
                return true;
            }
        }
        #endregion  
    }
    #endregion

}
